#include <pthread.h>
#include <string.h>
#include <stdlib.h>

#include "TaskManager.h"
#include "PeerHash.h"
#include "Network.h"

static pthread_mutex_t peerHashLock = PTHREAD_MUTEX_INITIALIZER;
struct PeerHash **peerHash;

int ph_add(struct PeerList *peer){
	if(peer->id < 0)
		return -1;
	pthread_mutex_lock(&peerHashLock);
	struct PeerHash *ptr = peerHash[peer->id % NPEERBUCKETS];
	struct PeerHash *last = NULL;

	while(ptr != NULL && ptr->id != peer->id){
		last = ptr;
		ptr = ptr->next;
	}
	
	if(ptr == NULL){
		ptr = malloc(sizeof *ptr);
		ptr->id = peer->id;
		ptr->peer = peer;
		ptr->waitingInstance = NULL;
		ptr->next = NULL;
		if(last == NULL)
			peerHash[peer->id % NPEERBUCKETS] = ptr;
		else
			last->next = ptr;
	}
	else{
		ptr->peer = peer;
		while(ptr->waitingInstance != NULL){
			struct InstanceStruct *instance = ptr->waitingInstance;
			ptr->waitingInstance = ptr->waitingInstance->next;
			if(instance->state == CHANRNW)
				instance->state = CHANR;
			else if(instance->state == CHANWNW)
				instance->state = CHANW;
			else
				instance->state = READY;
			tm_requeue(instance);
		}
	}
	pthread_mutex_unlock(&peerHashLock);
	return 0;
}

int ph_waitForNode(struct InstanceStruct *instance, int id){
	if(id < 0)
		return -1;
	pthread_mutex_lock(&peerHashLock);
	struct PeerHash *ptr = peerHash[id % NPEERBUCKETS];
	struct PeerHash *last = NULL;

	while(ptr != NULL && ptr->id != id){
		last = ptr;
		ptr = ptr->next;
	}
	
	if(ptr == NULL){
		ptr = malloc(sizeof *ptr);
		ptr->id = id;
		ptr->peer = NULL;
		ptr->waitingInstance = instance;
		ptr->next = NULL;
		if(last == NULL)
			peerHash[id % NPEERBUCKETS] = ptr;
		else
			last->next = ptr;
	}
	else if(ptr->peer != NULL){
		if(instance->state == CHANRNW)
			instance->state = CHANR;
		else if(instance->state == CHANWNW)
			instance->state = CHANW;
		else
			instance->state = READY;
		tm_requeue(instance);
	}
	else{
		instance->prev = NULL;
		instance->next = ptr->waitingInstance;
		if(ptr->waitingInstance != NULL){
			ptr->waitingInstance->prev = instance;
		}
		ptr->waitingInstance = instance;
	}
	pthread_mutex_unlock(&peerHashLock);
	return 0;
}

struct PeerList *ph_getPeer(int id){
	if(id < 0)
		return NULL;
	pthread_mutex_lock(&peerHashLock);
	struct PeerHash *ptr = peerHash[id % NPEERBUCKETS];
	while(ptr != NULL && ptr->id != id)
		ptr = ptr->next;
	
	pthread_mutex_unlock(&peerHashLock);
	if(ptr == NULL)
		return NULL;
	return ptr->peer;
}

int ph_init(){
	peerHash = malloc(NPEERBUCKETS * sizeof(struct PeerHash*));
	memset(peerHash, 0, NPEERBUCKETS * sizeof(struct PeerHash*));
	return 0;
}