#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "Global.h"
#include "Network.h"
#include "FunctionManager.h"
#include "TaskManager.h"
#include "PeerHash.h"
#include "ChanManager.h"


int main(int argc, char **argv){
	if(argc < 3){
		fprintf(stderr,ERRFIX"Not enough arguments!\n Usage: %s <id> <port> [-c <ip> <port>] [-r <function>]\n", argv[0]);
		return -1;
	}
	char *tmp1, *tmp2;
	int id = strtol(argv[1], &tmp1, 10);
	int port = strtol(argv[2], &tmp2, 10);
	if(*tmp1 != 0 || *tmp2 != 0){
		fprintf(stderr,ERRFIX"Id and port must be a numbers!\n Usage: %s <id> <port> [-c <ip> <port>] [-r <function>]\n", argv[0]);
		return -1;
	}
	if(port < 1025 || port > 65535){
		fprintf(stderr,ERRFIX"Port out of range! Valid range is 1025-65535\n Usage: %s <id> <port> [-c <ip> <port>] [-r <function>]\n", argv[0]);
		return -1;
	}
	if(id < 0 || id > 9999){
		fprintf(stderr,ERRFIX"Id out of range! Valid range is 0-9999\n Usage: %s <id> <port> [-c <ip> <port>] [-r <function>]\n", argv[0]);
		return -1;
	}
	
	
	if(tm_init(4)) //TODO dynamic set number of workers?
		goto errorLbl;
	if(fm_init())
		goto errorLbl;
	if(ph_init())
		goto errorLbl;
	if(ch_init(4)) //TODO dynamic set number of workers?
		goto errorLbl;
	
	
	char *mip = NULL;
	char *mpt = NULL;
	if(argc > 5 && strcmp(argv[3], "-c") == 0){
		mip = argv[4];
		mpt = argv[5];
	}
	
	int e = nw_init(id, port, mip, mpt);
	if(e == -2)
		goto panicLbl;
	if(e)
		goto errorLbl;
	
	char *prog = NULL;
	if(argc > 4 && strcmp(argv[3], "-r") == 0)
		prog = argv[4];
	else if(argc > 7 && strcmp(argv[6], "-r") == 0)
		prog = argv[7];
	
	if(prog != NULL){
		if(fm_loadFunction(prog) || fm_createInstance(prog, NULL))
			fprintf(stderr,ERRFIX"Could not run program!\n");
		else
			printf(PREFIX"Running program: %s\n", prog);
	}
	
	printf(PREFIX"Running\n");
	
	while(masterSwitch)
		sleep(1);
panicLbl:
	nw_panic();
errorLbl:
	sleep((argc > 4 ? 5 : 3)); //Sleep shorter if "master"
	printf(PREFIX"Restarting\n");
	char **argvv = malloc(sizeof(char**) * (argc + 1));
	int i;
	for(i = 0; i < argc; ++i)
		argvv[i] = argv[i];
	argvv[argc] = NULL;
	
	execve(argv[0], argvv, NULL);
	perror("Execve");
	
	return -1;
}